/*
 * Copyright (c) 2012 TradertoolsFX
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:

 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#property copyright "Copyright (c) 2012 TradertoolsFX"
#property link      "http://www.tradertools-fx.com"
#property description "This content is released under the terms of the MIT license (http://opensource.org/licenses/mit-license.html)."
#property show_inputs

#include <stdlib.mqh>

enum order_enum {
    BUY,
    SELL
};

// User Parameters
extern string _desc1 = "---Enter %Risk Per Trade---";
extern double RiskSetting = 2;
extern order_enum order_type = BUY;
extern string _desc2 = "---Enter SL as Fixed Pips OR Price---";
extern double StopLoss_Pips;
extern double StopLoss_Price;
extern string _desc3 = "---Enter Entry Price, Leave Blank to Use Current Price---";
extern double Entry_Price;
extern string _desc4 = "---Enter TP as Fixed Pips OR Price---";
extern double TakeProfit_Pips;
extern double TakeProfit_Price;

int init()
{
   if (Digits == 5 || Digits == 3) {
      StopLoss_Pips *= 10;
      TakeProfit_Pips *= 10;
   }

   return 0;
}

static double s_entry_price, s_sl_price, s_tp_price;

int init_entry_price(void)
{
    if (Entry_Price > 0) {
        // Use manually entered price parameter
        s_entry_price = Entry_Price;
    } else {
        // Use current market price
        s_entry_price = (order_type == BUY) ? Ask : Bid;
    }

    s_entry_price = NormalizeDouble(s_entry_price, Digits);

    // Success
    return 0;
}

int init_sl_price(void)
{
    if (!CompareDoubles(StopLoss_Pips, 0.0) &&
            !CompareDoubles(StopLoss_Price, 0.0)) {
        Alert("Invalid Parameters: BOTH stop loss pips and stop loss price are set. Choose only ONE!");
        return -1;
    } else if (CompareDoubles(StopLoss_Pips, 0.0) &&
               CompareDoubles(StopLoss_Price, 0.0)) {
        Alert("No stop loss parameter set!");
        return -1;
    }

    if (StopLoss_Price > 0) {
        s_sl_price = StopLoss_Price;
        // Check for invalid parameters
        if (order_type == BUY &&
                s_sl_price > s_entry_price) {
            Alert("Invalid Parameter: The stop loss price you entered for a BUY is greater than the entry price!");
            return -1;
        } else if (order_type == SELL &&
                       s_sl_price < s_entry_price) {
            Alert("Invalid Parameter: The stop loss price you entered for a SELL is less than the entry price!");
            return -1;
        }
    } else if (StopLoss_Pips > 0) {
        double sl_dist = StopLoss_Pips * Point;
        if (order_type == BUY) {
            s_sl_price = s_entry_price - sl_dist;
        } else if (order_type == SELL) {
            s_sl_price = s_entry_price + sl_dist;
        }
    }

    s_sl_price = NormalizeDouble(s_sl_price, Digits);

    // Success
    return 0;
}

int init_tp_price(void)
{
    if (!CompareDoubles(TakeProfit_Pips, 0.0) &&
            !CompareDoubles(TakeProfit_Price, 0.0)) {
        Alert("Invalid Parameters: BOTH take profit pips and take profit price are set. Choose only ONE!");
        return -1;
    }

    if (TakeProfit_Price > 0) {
        s_tp_price = TakeProfit_Price;
        // Check for invalid parameters
        if (order_type == BUY) {
            if (s_tp_price < s_entry_price) {
                Alert("Invalid Parameter: The take profit price you entered for a BUY is less than the entry price!");
                return -1;
            }
        } else if (order_type == SELL) {
            if (s_tp_price > s_entry_price) {
                Alert("Invalid Parameter: The take profit price you entered for a SELL is greater than the entry price!");
                return -1;
            }
        }
    } else if (TakeProfit_Pips > 0) {
        double tp_dist = TakeProfit_Pips * Point;
        if (order_type == BUY) {
            s_tp_price = s_entry_price + tp_dist;
        } else if (order_type == SELL) {
            s_tp_price = s_entry_price - tp_dist;
        }
    }

    s_tp_price = NormalizeDouble(s_tp_price, Digits);

    // Success
    return 0;
}

int start()
{
    if (init_entry_price() == -1 ||
            init_sl_price() == -1 ||
            init_tp_price() == -1) {
        return 0;
    }

    // Calculate and display risk stats
    double dollar_risk = dollar_risk(RiskSetting, AccountEquity());
    double lot_size = lot_size(RiskSetting,
                               dollar_risk,
                               price_to_pips(s_sl_price, s_entry_price));
    double reward_to_risk = 0.0;
    if (!CompareDoubles(s_tp_price, 0.0)) {
        reward_to_risk = 1 /
                         risk_to_reward(s_entry_price, s_sl_price, s_tp_price);
    }

    string order_type_str;
    if (order_type == BUY) {
        order_type_str = "BUY";
    } else if (order_type == SELL) {
        order_type_str = "SELL";
    }

    string risk_reward_str;
    if (CompareDoubles(s_tp_price, 0.0)) {
        risk_reward_str = "TP not set";
    } else {
        risk_reward_str = DoubleToStr(reward_to_risk, 0);
    }

    Alert("Risk: ", DoubleToStr(RiskSetting, 2), "%",
          " Monetary Risk: $", DoubleToStr(dollar_risk, 2),
          " Lot Size: ", DoubleToStr(lot_size, 2),
          " Risk-to-Reward: 1:", risk_reward_str, "\n",
          "Order Type: ", order_type_str,
          " Entry Price: ", s_entry_price,
          " Stop Loss: ", s_sl_price,
          " Take Profit: ", s_tp_price);

    return 0;
}

double dollar_risk(double risk_as_percent, double balance)
{
    double risk_as_decimal = risk_as_percent * 0.01;
    double risk_in_dollars = risk_as_decimal * balance;

    return NormalizeDouble(risk_in_dollars, 2);
}

double lot_size(double risk_as_percent, double dollar_risk, double sl_in_pips)
{
    double dollar_risk_per_pip = dollar_risk / sl_in_pips;
    // Current tick value in deposit currency for 1.0 lots
    double tick_value = MarketInfo(Symbol(), MODE_TICKVALUE);
    double lot_size = dollar_risk_per_pip / tick_value;

    return NormalizeDouble(lot_size, 2);
}

double risk_to_reward(double entry_price, double sl_price, double tp_price)
{
    double sl_dist = MathAbs(entry_price - sl_price);
    double tp_dist = MathAbs(entry_price - tp_price);

    double rr_ratio = sl_dist / tp_dist;

    return rr_ratio;
}

double price_to_pips(double price_to_convert, double entry_price)
{
    double price_difference = MathAbs(price_to_convert - entry_price);
    double pips = price_difference / Point;

    return pips;
}
